import {
  ConflictException,
  ForbiddenException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { CarreraPrograma } from './entity/carrera-programa.entity';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { InstitucionProgramaService } from '../institucion-programa/institucion-programa.service';

@Injectable()
export class CarreraProgramaService {
  constructor(
    @InjectRepository(CarreraPrograma)
    private repository: Repository<CarreraPrograma>,
    private institucionService: InstitucionService,
    private institucionCarreraService: InstitucionCarreraService,
    private institucionProgramaService: InstitucionProgramaService,
  ) {}

  async create(
    admin: Operador,
    id_institucion_carrera: number,
    id_programa: number,
  ) {
    const institucionCarrera = await this.institucionCarreraService.findById(
      id_institucion_carrera,
    );
    const programa = await this.institucionProgramaService.findProgramaById(
      id_programa,
    );

    if (
      admin.institucion.id_institucion !=
      institucionCarrera.institucion.id_institucion
    )
      throw new ForbiddenException(
        'Esta carrera no pertenece a tu institución.',
      );
    return this.repository
      .findOne({ where: { institucionCarrera, programa } })
      .then((existeCarretaPrograma) => {
        if (existeCarretaPrograma)
          throw new ConflictException(
            'Ya se asignó este programa a esta carrera.',
          );
        return this.repository.save(
          this.repository.create({
            institucionCarrera,
            programa,
          }),
        );
      })
      .then((_) => ({
        message: 'Se asignó correctamente el programa a la carrera.',
      }));
  }

  delete(admin: Operador, id_carrera_programa: number) {
    return this.findById(id_carrera_programa)
      .then((carreraPrograma) => {
        if (
          admin.institucion.id_institucion !=
          carreraPrograma.institucionCarrera.institucion.id_institucion
        )
          throw new ForbiddenException(
            'No puedes eliminar esta asociación porque no pertenece a tu institución.',
          );
        return this.repository.delete(carreraPrograma);
      })
      .then((_) => ({
        message: 'Se eliminó correctamente la asignación del programa.',
      }));
  }

  findById(id_carrera_programa: number) {
    return this.repository
      .findOne({ where: { id_carrera_programa } })
      .then((carreraPrograma) => {
        if (!carreraPrograma)
          throw new NotFoundException('No existe este id carrera programa.');
        return carreraPrograma;
      });
  }

  findByIdInstitucion(
    usuarioOperador: Operador | Usuario,
    id_institucion: number,
  ) {
    return this.institucionService
      .findInfoInstitucionById(id_institucion)
      .then((institucion) => {
        if (usuarioOperador instanceof Operador) {
          if (
            usuarioOperador.institucion.id_institucion !=
            institucion.id_institucion
          )
            throw new ForbiddenException(
              'No puedes acceder a esta información porque no perteneces a esta institución.',
            );
        } else {
          let lePertenece = false;

          for (let i = 0; i < usuarioOperador.instituciones.length; i++)
            if (
              usuarioOperador.instituciones[i].institucionCarrera.institucion
                .id_institucion === institucion.id_institucion
            ) {
              lePertenece = true;
              break;
            }
          if (!lePertenece)
            throw new ForbiddenException(
              'No puedes acceder a esta información porque no perteneces a esta institución.',
            );
        }
        return this.repository.find({
          join: {
            alias: 'cp',
            innerJoinAndSelect: {
              ic: 'cp.institucionCarrera',
              c: 'ic.carrera',
              i: 'ic.institucion',
            },
          },
          where: { institucionCarrera: { institucion } },
        });
      });
  }
}
