import * as moment from 'moment';
import {
  BadRequestException,
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionDia } from './entity/institucion-dia.entity';
import { Operador } from '../operador/entity/operador.entity';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionDiaService {
  constructor(
    @InjectRepository(InstitucionDia)
    private repository: Repository<InstitucionDia>,
    private institucionService: InstitucionService,
  ) {}

  findAllByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findInfoInstitucionById(id_institucion)
      .then((institucion) =>
        this.repository.find({
          where: { institucion },
          order: { id_institucion_dia: 'ASC' },
        }),
      );
  }

  findById(id_institucion_dia: number) {
    return this.repository
      .findOne({
        join: {
          alias: 'id',
          innerJoinAndSelect: {
            d: 'id.dia',
            i: 'id.institucion',
          },
        },
        where: { id_institucion_dia },
      })
      .then((institucionDia) => {
        if (!institucionDia)
          throw new NotFoundException('No existe este id institución día.');
        return institucionDia;
      });
  }

  findByInstitucionDia(institucion: Institucion, id_dia: number) {
    return this.repository.findOne({ where: { institucion, dia: { id_dia } } });
  }

  async hoy(id_institucion: number) {
    const ahora = moment();

    if (ahora.weekday() === 0)
      throw new ConflictException('No hay servicio los días domingo.');
    return this.institucionService
      .findInfoInstitucionById(id_institucion)
      .then((institucion) =>
        this.findByInstitucionDia(institucion, ahora.weekday()),
      );
  }

  update(admin: Operador, attrs: Partial<InstitucionDia>) {
    return this.findById(attrs.id_institucion_dia)
      .then((institucionDia) => {
        if (
          admin.institucion.id_institucion !=
          institucionDia.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes actualizar la información de este día porque no le pertenece a tu institución.',
          );

        if (
          (attrs.hora_inicio || attrs.hora_fin) &&
          (attrs.hora_inicio || institucionDia.hora_inicio) >
            (attrs.hora_fin || institucionDia.hora_fin)
        )
          throw new BadRequestException(
            'La hora inicio no puede ser mayor que la hora fin.',
          );
        if (
          (attrs.hora_fin || attrs.hora_extra) &&
          (attrs.hora_fin || institucionDia.hora_fin) >
            (attrs.hora_extra || institucionDia.hora_extra)
        )
          throw new BadRequestException(
            'La hora fin no puede ser mayor que la hora extra.',
          );
        if (
          (attrs.hora_extra || attrs.hora_tope) &&
          (attrs.hora_extra || institucionDia.hora_extra) >
            (attrs.hora_tope || institucionDia.hora_tope)
        )
          throw new BadRequestException(
            'La hora extra no puede ser mayor que la hora tope.',
          );
        Object.assign(institucionDia, attrs);
        return this.repository.save(institucionDia);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
